unit Curve;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, sysUtils, Messages,
  clipBrd, spin, Gauges, math,
  jmmUtil, Tds200DDE, Main, OscConst,
  adExcept, adPort;

const
      MaxVecteurOsc = 2500;
      voieX = 0;
      voieA = 1;
      voieB = 2;

type
  indiceVecteurOsc = 0..MaxVecteurOsc;
  VecteurOsc = array[indiceVecteurOsc] of integer;
  indiceVoie = voieX..voieB;
  Tvoie = class
       Nom,Unite : string;
       Signif : String;
       Active : boolean;
       Gauge : Tgauge;
       Valeur : VecteurOsc;
       increment,offset,zero : extended;
       precision : integer;
       constructor create;
       function ValeurR(j : indiceVecteurOsc) : extended;
       function StrValeurPoint(j : indiceVecteurOsc) : string;
  end;

  TOscilloForm = class(TForm)
    SaveDialog: TSaveDialog;
    BoutonsPanel: TPanel;
    FileSaveBtn: TSpeedButton;
    ClipSaveBtn: TSpeedButton;
    RegSaveBtn: TSpeedButton;
    EnregistreBtn: TSpeedButton;
    GroupBox1: TGroupBox;
    EditNomB: TEdit;
    Label1: TLabel;
    EditCommB: TEdit;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    EditNomA: TEdit;
    EditCommA: TEdit;
    GaugeA: TGauge;
    GaugeB: TGauge;
    Label7: TLabel;
    StopBtn: TSpeedButton;
    XYBtn: TCheckBox;
    ExitBtn: TSpeedButton;
    procedure ExitItemClick(Sender: TObject);
    procedure SaveItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EnregistreBtnClick(Sender: TObject);
    procedure RegressiItemClick(Sender: TObject);
    procedure RegSaveBtnClick(Sender: TObject);
    procedure ClipSaveBtnClick(Sender: TObject);
    procedure SaveData(const NomFichier : String);
    procedure TriggerAvail(CP: TObject; Count: Word);
    procedure EditNomChange(Sender: TObject);
    procedure EditCommChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure RegNewFileClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ExitBtnClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    NbreCourant,NbreCarac : integer;
    VoieCourante : indiceVoie;
    AcqFaite,AcqAsauver : boolean;
    LectureVoie : Tlecture;
    ChaineOsc : ansiString;
    Procedure SetBoutonAcq(Permis : boolean);
    procedure LitVoie(Avoie : indiceVoie);
    procedure ChaineAvail(chaine : ansiString);
  public
     Voie : array[indiceVoie] of Tvoie;
     NbrePoints : integer;
  protected
     procedure WMGetMinMaxInfo(var Msg: TWMGetMinMaxInfo); message wm_GetMinMaxInfo;
  end;

var
  OscilloForm: TOscilloForm;

implementation

{$R *.DFM}

procedure TOscilloForm.ExitItemClick(Sender: TObject);
begin
    Close
end;

procedure TOscilloForm.SaveItemClick(Sender: TObject);
begin
     if SaveDialog.Execute then SaveData(saveDialog.FileName)
end;

procedure TOscilloForm.FormCreate(Sender: TObject);
var v : indiceVoie;
    j : integer;
begin
  AcqFaite := false;
  AcqAsauver := false;
  LectureVoie := lectNone;
  for v := voieX to voieB do
      voie[v] := Tvoie.create;

  voie[voieA].nom := 'V1';
  editNomA.text := 'V1';
  voie[voieA].gauge := gaugeA;

  voie[voieB].nom := 'V2';
  editNomB.text := 'V2';
  voie[voieB].gauge := gaugeB;

  voie[voieX].unite := 's';
  voie[voieX].signif := 'temps';
  voie[voieX].nom := 't';
  voie[voieX].precision := precisionMax;
  voie[voieX].active := true;
  for j := 0 to maxVecteurOsc do
      voie[voieX].valeur[j] := j-1;

end;

procedure TOscilloForm.LitVoie(Avoie : indiceVoie);
var codeTDS340 : string;
    buffer : string;

Function GetValeur(commande : ansiString) : double;
var posdp : integer;
begin
     if FormMain.isTDS340 then begin
        posdp := pos(':',commande);
        insert(codeTDS340,commande,posdp+1);
     end;
     result := FormMain.GetFloat(commande);
     if (result=NombreErreur) or
        (result=NombreTimeOut) or
        isNan(result) then raise EgetFloatError.create(commande);
end;

begin
     try
     voieCourante := Avoie;
     voie[Avoie].active := false;
     codeTDS340 := 'CH'+intToStr(aVoie)+':';
     voie[Avoie].Gauge.maxValue := 2500;
     FormMain.EnvoieCommande(SetSourceOsc+intToStr(aVoie));
     DelaySec(1);
     voie[voieX].offset := 0;
     voie[voieX].Increment := GetValeur(Xinc);
     voie[Avoie].Increment := GetValeur(Yinc);
     voie[Avoie].Offset := GetValeur(Yoffset);
     voie[Avoie].zero := GetValeur(Yzero);
     NbreCourant := 0;
     AcqFaite := false;
     LectureVoie := lectDiese;
     if FormMain.oscillo=oSerie then begin
        FormMain.VoieSerie.OnTriggerAvail := TriggerAvail;
        FormMain.onChaineAvail := ChaineAvail;
     end;
     FormMain.EnvoieCommande(GetData);
     arretLecture := false;
     stopBtn.Visible := true;
     repeat
         if (NbreCourant-voie[Avoie].Gauge.progress)>(NbrePoints div 32) then
            voie[Avoie].Gauge.progress := NbreCourant;
            FormMain.VoieSerie.processCommunications;
         Application.processMessages;
         if NbreCourant>=NbrePoints then begin
             AcqFaite := true;
             lectureVoie := lectNone;
         end;
     until AcqFaite or arretLecture;
     LectureVoie := lectNone;
     stopBtn.Visible := false;
     if FormMain.oscillo=oSerie then begin
        FormMain.VoieSerie.OnTriggerAvail := nil;
        FormMain.onChaineAvail := nil;
     end;
     NbreCourant := 0;
     if AcqFaite
       then begin
           voie[Avoie].Gauge.progress := NbrePoints;
           voie[Avoie].active := true;
       end
       else showMessage('Acquisition de CH'+IntToStr(Avoie)+' non effectue par l''ordinateur');
     except
        on EgetFloatError do begin
            showMessage('Impossible de trouver les calibres');
        end;
     end;  
end;

procedure TOscilloForm.EnregistreBtnClick(Sender: TObject);
var v : indiceVoie;
    reponse : string;
    modeXY : boolean;
begin
     AcqFaite := false;
     try
     setBoutonAcq(false);
     for v := voieA to voieB do voie[v].Gauge.progress := 0;
     modeXY := pos('XY',FormMain.getString('DIS:FORM?'))>0;
     if modeXY then begin
        FormMain.EnvoieCommande('DIS:FORM YT');
        xyBtn.checked := true;
     end;
     for v := voieA to voieB do begin
         reponse := FormMain.GetString(CHactive+intToStr(v)+'?');
         voie[v].active := (pos('ON',reponse)>0) or (pos('1',reponse)>0);
     end;
     for v := voieA to voieB do
         if voie[v].active then LitVoie(v);
     if modeXY then FormMain.EnvoieCommande('DIS:FORM XY');
     setBoutonAcq(true);
     AcqAsauver := AcqFaite;
     except
        setBoutonAcq(true);
        AcqAsauver := false;
     end;
end;

procedure TOscilloForm.RegressiItemClick(Sender: TObject);
begin
     FormDDE.Execute('FOCUS()')
end;

procedure TOscilloForm.RegSaveBtnClick(Sender: TObject);
begin
     if ConstanteDlg.showModal=mrOk then begin
        FormDDE.EnvoieDonneesOscillo(true,XYBtn.checked);
        AcqAsauver := false;
     end;
end;

procedure TOscilloForm.ClipSaveBtnClick(Sender: TObject);
begin
     FormDDE.EnvoieDonneesOscillo(false,XYBtn.checked);
     AcqAsauver := false;
end;

procedure TOscilloForm.SetBoutonAcq(permis : boolean);
begin
     FileSaveBtn.Visible := permis;
     ClipSaveBtn.Visible := permis;
     RegSaveBtn.Visible := permis;
     FileSaveBtn.enabled := AcqFaite;
     ClipSaveBtn.enabled := AcqFaite;
     RegSaveBtn.enabled := AcqFaite;
     StopBtn.visible := false;
     if permis
        then screen.cursor := crDefault
        else screen.cursor := crHourGlass;
end;

procedure TOscilloForm.SaveData(const NomFichier : String);
var fichier : textFile;
    formatRW3 : boolean;
    NbreVoies : integer;

Procedure EcritPage;
var j : indiceVecteurOsc;
    v : indiceVoie;
begin if formatRW3
  then begin
     writeln(fichier,SymbReg,'1 PAGE COMMENT');
     writeln(fichier,'');
     writeln(fichier,SymbReg2,NbrePoints,' VALEUR VAR');
     for j := 1 to NbrePoints do begin
         for v := voieX to voieB do with voie[v] do if active then
             write(fichier,StrValeurPoint(j),' ');
         writeln(fichier);
     end
 end
 else begin
    for j := 1 to NbrePoints do begin
        for v := voieX to voieB do with voie[v] do
            if active then
               write(fichier,StrValeurPoint(j),crTab);
        writeln(fichier);
    end;
end
end; // ecritPage

Procedure ecritEnTete;
var j : indiceVoie;
begin if formatRW3
then begin
   writeln(fichier,'EVARISTE REGRESSI WINDOWS 1.0');
   writeln(fichier,symbReg,NbreVoies,' NOM VAR');
   for j := voieX to voieB do if voie[j].active then
       writeln(fichier,voie[j].nom);
   writeln(fichier,symbReg,NbreVoies,' UNITE VAR');
   for j := voieX to VoieB do if voie[j].active then
      writeln(fichier,voie[j].unite);
   writeln(fichier,symbReg,NbreVoies,' SIGNIF VAR');
   for j := voieX to VoieB do if voie[j].active then
      writeln(fichier,voie[j].signif);
   writeln(fichier,symbReg,'1 ACQUISITION');
   writeln(fichier,Application.ExeName)
end
else begin
   writeln(fichier,'TDS200');
   for j := voieX to VoieB do if voie[j].active then
      write(fichier,voie[j].nom,crTab);
   writeln(fichier);
   for j := voieX to VoieB do if voie[j].active then
      write(fichier,voie[j].unite,crTab);
   writeln(fichier);
   for j := voieX to VoieB do if voie[j].active then
      write(fichier,voie[j].signif,crTab);
   writeln(fichier);
end
end; { ecritEnTete }

var v : indiceVoie;
begin
    Screen.Cursor := crHourGlass;
    FormatRW3 := AnsiUpperCase(ExtractFileExt(NomFichier))='.RW3';
    AssignFile(fichier,NomFichier);
    Rewrite(fichier);
    NbreVoies := 0;
    for v := voieX to voieB do
        if voie[v].active then inc(NbreVoies);
    ecritEnTete;
    ecritPage;
    CloseFile(fichier);
    Screen.Cursor := crDefault;
    AcqAsauver := false;
end;

procedure TOscilloForm.TriggerAvail(CP: TObject; Count: Word);
var i : integer;
    carac : ansiChar;
begin
   for i := 1 to count do begin
      carac := FormMain.voieSerie.GetChar;
// <header> = CURVe #n<yyyy> n tant le nombre de y
      case LectureVoie of
           lectDiese : if carac='#' then lectureVoie := lectNcarac;
           lectNcarac : begin
              NbreCarac := StrToInt(carac); // 4 ?
              NbreCourant := 0;
              ChaineOsc := '';
              lectureVoie := lectNbyte;
           end;
           lectNbyte : begin
               inc(NbreCourant);
               chaineOsc := chaineOsc+carac;
               if NbreCourant=NbreCarac then begin
                  NbrePoints := strToInt(chaineOsc); // 2500 ?
                  lectureVoie := lectData;
                  NbreCourant := 0;
                  voie[voieCourante].Gauge.maxValue := NbrePoints;
               end;
           end;
           lectData : begin
                inc(NbreCourant);
                voie[voieCourante].valeur[NbreCourant] := ord(carac);
            end;
       end
   end;
end;

procedure TOscilloForm.ChaineAvail(chaine: ansiString);
var i : integer;
    carac : ansiChar;
begin
   for i := 1 to length(chaine) do begin
      carac := chaine[i];
//  <header> = CURVe #n<yyyy> n tant le nombre de y
      case LectureVoie of
           lectDiese : if carac='#' then lectureVoie := lectNcarac;
           lectNcarac : begin
              NbreCarac := StrToInt(carac); { 4 ? }
              NbreCourant := 0;
              ChaineOsc := '';
              lectureVoie := lectNbyte;
           end;
           lectNbyte : begin
               inc(NbreCourant);
               chaineOsc := chaineOsc+carac;
               if NbreCourant=NbreCarac then begin
                  NbrePoints := strToInt(chaineOsc);  { 2500 ? }
                  lectureVoie := lectData;
                  NbreCourant := 0;
                  voie[voieCourante].Gauge.maxValue := NbrePoints;
               end;
           end;
           lectData : begin
                inc(NbreCourant);
                voie[voieCourante].valeur[NbreCourant] := ord(carac);
            end;
       end
   end;
end;

procedure TOscilloForm.EditNomChange(Sender: TObject);
var NumVoie : indiceVoie;
begin
     NumVoie := (sender as Tcomponent).tag;
     voie[NumVoie].nom := (sender as Tedit).text;
end;

procedure TOscilloForm.EditCommChange(Sender: TObject);
var NumVoie : indiceVoie;
begin
     NumVoie := (sender as Tcomponent).tag;
     voie[NumVoie].signif := (sender as Tedit).text;
end;

constructor Tvoie.create;
begin
    inherited create;
    signif := 'tension';
    active := true;
    unite := 'V';
    precision := PrecisionMin;
end;

function Tvoie.valeurR(j : indiceVecteurOsc) : extended;
begin
    valeurR := (valeur[j]-offset)*Increment+zero
end;

function Tvoie.strValeurPoint(j : indiceVecteurOsc) : string;
begin
     result := FloatToStrPoint(valeurR(j))
end;

procedure TOscilloForm.FormActivate(Sender: TObject);
begin
     setBoutonAcq(true)
end;

procedure TOscilloForm.RegNewFileClick(Sender: TObject);
begin
     FormDDE.EnvoieDonneesOscillo(true,XYBtn.checked)
end;

procedure TOscilloForm.FormDestroy(Sender: TObject);
var v : indiceVoie;
begin
     for v := voieX to voieB do Voie[v].free;
     OscilloForm := nil
end;

procedure TOscilloForm.WMGetMinMaxInfo(var Msg: TWMGetMinMaxInfo);
begin
     with Msg.MinMaxInfo^ do begin
          ptMinTrackSize := point(280,240);
          ptMaxTrackSize := point(screen.width-20,240);
     end;
end;

procedure TOscilloForm.FormResize(Sender: TObject);
begin
     EditCommA.width := GroupBox1.width-EditCommA.left-4;
     EditCommB.width := GroupBox2.width-EditCommB.left-4;
end;

procedure TOscilloForm.StopBtnClick(Sender: TObject);
begin
     ArretLecture := true
end;

procedure TOscilloForm.FormClose(Sender: TObject;var Action: TCloseAction);
begin
  if AcqAsauver and
     (MessageDlg('Sauvegarde',mtConfirmation,[mbYes,mbNo],0)=idYes)
     then SaveItemClick(sender);
  FormMain.setMode(mNone);
  Action := caFree;
end;


procedure TOscilloForm.ExitBtnClick(Sender: TObject);
begin
   close
end;

procedure TOscilloForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
     if (key=' ') and enregistreBtn.visible then begin
         enregistreBtnClick(sender);
         key := #0;
     end;
end;

end.


