unit Jmmutil;

interface

uses
   SysUtils, Windows, Messages, Classes, Dialogs, Forms,
   StdCtrls, Consts, math, oomisc, awUser, awWin32, shlObj;

const
   enDegre: boolean = True;
   NombreErreur: double = 1.2e37;
   NombreTimeOut: double = 1.3e37;
   ProcheReal: double = 1e-6;
   PrecisionMin = 5;
   PrecisionMax = 7; { Precision double pour FloatToStrF }
   SymbReg: char = '';
   SymbReg2: char = '&';
   Tabulation = #9;
   Loge       = 0.434294481903251828;
   Ln10       = 2.302585092994045680;

   Xinc          = 'WFMP:XIN?';
   Xzero         = 'WFMP:XZE?';
   Yinc          = 'WFMP:YMU?';
   Yoffset       = 'WFMP:YOF?';
   Yzero         = 'WFMP:YZE?';
   GetData       = 'CURV?';
   SetSourceOsc  = 'DAT:SOU CH';
   CHactive      = 'SEL:CH';
   GetTimeDiv    = 'HOR:SCA?';
   SetSourceMult = 'SOU CH';
   SetType       = 'TYP';
   GetValeurStr  = 'VAL?';
   EnTeteMesure  = 'MEASU:IMM:';

type
   Tlecture     = (LectNone, LectDiese, LectNcarac, LectNByte, LectData);
   ChaineAzt    = array[0..255] of char;
   CoefUnite = (zero, yocto, zepto, atto, femto, pico, nano, micro, milli,
      nulle, kilo, mega, giga, tera, peta, exa, zetta, yotta, infini);

const
   RadianVersDegre = 57.29577951308232090;
   Majuscule: TsyscharSet = ['A'..'Z'];
   Minuscule: TsyscharSet = ['a'..'z'];
   Chiffre: TsyscharSet = ['0'..'9'];
   ChiffreEtc: TsyscharSet = ['0'..'9', '+', '-', 'e', 'E', '.'];
   ChiffreWin: TsyscharSet = ['0'..'9', '+', '-', 'e', 'E', '.', ','];
   crTab      = char(vk_tab);
   crSupprArr = char(vk_back);
   crHaut     = char(vk_up);
   crBas      = char(vk_down);
   crGauche   = char(vk_left);
   crDroite   = char(vk_right);
   crHome     = char(vk_home);
   crEnd      = char(vk_end);
   crCR       = #13;
   crLF       = #10;
   crESC      = #27;
   crEOT      = #4;
   crXon      = #17;
   crXoff     = #19;
   symboleUnite: array[coefUnite] of char =
      ('0', 'y', 'z', 'a', 'f', 'p', 'n', '', 'm', ' ', 'k', 'M', 'G',
      'T', 'P', 'E', 'Z', 'Y', '');
   valUnite: array[coefUnite] of real =
      (1E-27, 1E-24, 1E-21, 1E-18, 1E-15, 1E-12, 1E-9, 1E-6, 1E-3, 1,
      1E+3, 1E+6, 1E+9, 1E+12, 1E+15, 1E+18, 1E+21, 1E+24, 1E+27);
   separateur: TsysCharSet   = [' ', crTab, ','];
   caracPrefixe: TsyscharSet =
      ['y', 'z', 'a', 'f', 'p', 'n', '', 'm', 'k', 'M', 'G', 'T', 'P', 'E'];

function Pad(S: string; Len: byte): string;
procedure Validatedouble(Sender: Tedit; var z: double);
procedure VerifKeyNum(var key: char);
function ConvertitChaine(nombre: string; var valeur: double): boolean;
function FormatCourt(nombre: double; NomUnite: string): string;
function FloatToStrPoint(valeur: extended): string;
function StrToFloatWin(nombre: string): double;
function DureeEcoulee(InstantDebut: TdateTime): double;
function EncodeDuree(duree: longint): TdateTime;
procedure DelaySec(s: double);
function Log(x: extended): extended;
function IntMaths(x: extended): extended;
function OK(texte: String): boolean;
procedure Erreur(texte: String);
function IsPortAvailable(ComNum: cardinal): boolean;
function Atan2(im, re: double): double;
function DixReel(Y: double): double; { 10^Y }

var
   mesDocsDir: string;
   NomFichierIni: string;

implementation

Function StrToFloatWin(nombre : String) : double;
var posSeparator : integer;
begin
   if decimalSeparator<>'.' then begin
   posSeparator := pos('.',nombre);
   if posSeparator>0 then Nombre[posSeparator] := decimalSeparator;
   end
   else if decimalSeparator<>',' then begin
   posSeparator := pos(',',nombre);
   if posSeparator>0 then Nombre[posSeparator] := decimalSeparator;
   end;
   StrToFloatWin :=  strToFloat(nombre);
end;

function FloatToStrPoint(valeur: extended): string;
var
   nombre: string;
   posSeparator: byte;
begin
   nombre := FloatToStrF(valeur, ffGeneral, 7, 1);
   if decimalSeparator <> '.' then begin
      posSeparator := pos('.', nombre);
      if posSeparator > 0 then
         Nombre[posSeparator] := '.';
   end;
   Result := nombre;
end;

function ConvertitChaine(nombre: string; var valeur: double): boolean;
begin
   if (upcase(nombre[1]) = 'E') then
      nombre := '1' + nombre
   else if nombre[1] = '.' then
      nombre := '0' + nombre;
   convertitChaine := True;
   try
      valeur := StrToFloat(nombre);
   except
      convertitChaine := False;
   end;
end;

function FormatCourt(Nombre: double; NomUnite: string): string;
var
   unite:  coefUnite;
   Negatif: boolean;
   tampon: string;
begin
   if isNan(nombre) then
      FormatCourt := ''
   else if nombre = NombreErreur then
      FormatCourt := 'erreur'
   else if nombre = NombreTimeOut then
      FormatCourt := 'time-out'
   else begin
      negatif := nombre < 0;
      nombre := abs(nombre);
      unite := nulle;
      while (nombre >= 999) and (unite < infini) do begin
         nombre := nombre / 1000;
         Inc(unite);
      end; { nombre<1000 ou infini }
      while (nombre < 0.999) and (unite > zero) do begin
         nombre := nombre * 1000;
         Dec(unite);
      end; { nombre>=1 ou zero }
      if unite in [infini, zero] then
         tampon := '0 '
      else begin
         tampon := FloatToStrF(nombre, ffGeneral, PrecisionMin, 0);
         if negatif then
            tampon := '-' + tampon;
         tampon := tampon + ' ';
         if unite <> nulle then
            tampon := tampon + symboleUnite[unite];
      end;
      FormatCourt := tampon + NomUnite;
   end;
end;

function ToucheValidation(key: word): boolean;
begin
   ToucheValidation := key in [vk_tab, vk_left, vk_right, vk_down, vk_up, vk_return];
end;

procedure VerifKeyNum(var key: char);
begin
   if not charInset(key,ChiffreEtc + [crSupprArr]) then
      key := #0;
end;

procedure Validatedouble(Sender: Tedit; var z: double);
begin
   try
      z := StrToFloat(Sender.Text)
   except
      Sender.Text := FloatToStr(z);
      raise
   end;
end;

function DureeEcoulee(InstantDebut: TdateTime): double;
var
   h, m, s, ms: word;
begin
   decodeTime(time - instantDebut, h, m, s, ms);
   DureeEcoulee := h * 3600.0 + m * 60.0 + s * 1.0 + ms * 0.001;
end;

function Pad(S: string; Len: byte): string;
 //  Return a string right-padded to length len with ' '
var
   SLen: integer;
   I: Integer;
begin
   Slen := Length(S);
   if SLen >= Len then begin
      result := copy(S, 1, Len);
      exit;
   end;
   for I := Slen+1 to Len do
       s := s + ' ';
   Pad := s;
end;

function EncodeDuree(duree: longint): TdateTime;
var
   h, m, s, ms: word;
begin
   h  := duree div 3600;
   duree := duree - h * 3600;
   m  := duree div 60;
   duree := duree - m * 60;
   s  := duree;
   ms := 0;
   EncodeDuree := EncodeTime(h, m, s, ms);
end;

procedure DelaySec(s: double);
var
   InstantDebut: TdateTime;
begin
   InstantDebut := Time;
   repeat
   until (DureeEcoulee(InstantDebut) > s);
end;

function Log(X: extended): extended;
begin
   Log := Ln(X) * loge;
end;

function PuissEnt(X: extended; N: integer): extended;

   function PuissDeux(N: integer): extended; { N>=0 }
   var
      Y: extended;
   begin
      if N = 0 then
         PuissDeux := 1
      else begin
         Y := PuissDeux(N div 2);
         Y := Sqr(Y);
         if (N mod 2) = 0 then
            PuissDeux := Y
         else
            PuissDeux := X * Y;
      end;
   end;{ puissDeux }

begin {PuissEnt}
   if N < 0 then
      PuissEnt := 1 / PuissDeux(-N)
   else
      PuissEnt := PuissDeux(N);
end;{PuissEnt}

function Dix(x: integer): extended;
begin
   Dix := PuissEnt(10, X);
end;

function Plafond(x: extended): longint;
var
   zut: longint;
begin
   Zut := trunc(x);
   if (x < 0) or (Zut = x) then
      Plafond := zut
   else
      Plafond := zut + 1;
end;

function IntMaths(x: extended): extended;
   { "vraie" partie entire }
var
   zut: extended;
begin
   Zut := int(x);
   if (x >= 0) or (zut = x) then
      intMaths := zut
   else
      intMaths := zut - 1;
end;

function OK(texte: String): boolean;
begin
   OK := MessageDlg(texte, mtConfirmation, [mbYes, mbNo], 0) = idYes;
end;

procedure Erreur(texte: String);
begin
   MessageDlg(texte, mtInformation, [mbOK], 0);
end;

function IsPortAvailable(ComNum: cardinal): boolean;

   function MakeComName(const Dest: PChar; const ComNum: cardinal): PChar;
   begin
      StrFmt(Dest, '\\.\COM%d', [ComNum]);
      MakeComName := Dest;
   end;

var
   ComName: array[0..12] of char;
   Res: integer;
   DeviceLayer: TApdBaseDispatcher;
begin
   DeviceLayer := TApdWin32Dispatcher.Create(nil);
   try
      if (ComNum = 0) then
         Result := False
      else
      begin
         Res := DeviceLayer.OpenCom(MakeComName(ComName, ComNum), 64, 64);
         Result := (Res >= 0) or (GetLastError =
            DWORD(Abs(ecAccessDenied)));
         if Result then
            DeviceLayer.CloseCom;
      end;
   finally
      DeviceLayer.Free;
   end;
end;

function Atan2(im, re: double): double;
   { z = re + j*im }
begin
   try
      Result := arcTan(im / re);
      if re < 0 then
         if im < 0 then
            Result := Result - pi
         else
            Result := Result + pi;
   except
      on EZeroDivide do begin
         if im < 0 then
            Result := -pi / 2
         else
            Result := pi / 2;
      end;
   end;
   if EnDegre then
      Result := Result * RadianVersDegre;
end;

function DixReel(Y: double): double; { 10^Y }
var
   Z: integer;
begin
   Z := round(Y);
   if Abs(Y - Z) < ProcheReal then
      Result := PuissEnt(10, Z)
   else
      Result := Exp(Ln(10) * Y);
end;{DixReel}

function AddBackSlash(aPath: string): string;
begin
   Result := EmptyStr;
   if (aPath <> '') and (aPath[Length(aPath)] <> '\') then
      Result := aPath + '\';
end;

// permet de retrouver un repertoire d'aprs sa valeur CSIDL
function GetFolder(CSIDLValue: integer): string;
var
   IdList: PITEMIDLIST;
   Folder: array[0..MAX_PATH] of char;
begin
   if not Failed(SHGetSpecialFolderLocation(0, CSIDLValue, IdList)) then begin
      SHGetPathFromIDList(IdList, Folder);
      Result := AddBackSlash(Folder);
   end;
end;

initialization
   mesDocsDir := GetFolder(CSIDL_Personal);
   mesDocsDir := MesDocsDir + 'Regressi';
   if not DirectoryExists(MesDocsDir) then
      createDir(MesDocsDir);
   mesDocsDir := MesDocsDir + '\';
   NomFichierIni := ChangeFileExt(Application.exeName, '.ini');
end.

