unit Main;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
   Dialogs, StdCtrls, Buttons, ExtCtrls, SysUtils, Messages,
   clipBrd, spin, inifiles, XPMan, Variants,
   jmmUtil, adexcept, OoMisc, adPort;

type
   EserieError = class(Exception);
   EgetFloatError = class(Exception);
   Tmode        = (mNone, mOsc, mMult, mBode);
   TchaineAvail = procedure(chaineLue: ansiString) of object;
   Toscillo = (oNone,oSerie);

   TFormMain = class(TForm)
      BoutonPanel: TPanel;
      OscBtn:      TBitBtn;
      ComBtn:      TBitBtn;
      ExitBtn:     TBitBtn;
      AboutBtn:    TBitBtn;
      AidePanel:   TPanel;
      VoieSerie:   TApdComPort;
      MultBtn:     TBitBtn;
      BodeBtn:     TBitBtn;
      procedure ExitItemClick(Sender: TObject);
      procedure AboutItemClick(Sender: TObject);
      procedure FormCreate(Sender: TObject);
      procedure FormClose(Sender: TObject; var Action: TCloseAction);
      procedure ResetOscilloClick(Sender: TObject);
      procedure FormActivate(Sender: TObject);
      procedure OscBtnClick(Sender: TObject);
      procedure MultBtnClick(Sender: TObject);
      procedure BodeBtnClick(Sender: TObject);
   private
      function rs232Present : boolean;
      procedure AppActivate(Sender: TObject);
      procedure ShowHint(Sender: TObject);
   public
      isTDS340: boolean;
      Oscillo : Toscillo;
      IdentiteScope, NomScope: string;
      AcquisitionEnCours: Tmode;
      OnChaineAvail: TChaineAvail;
      procedure ResetOscillo;
      function GetString(const commande: ansiString): ansiString;
      procedure EnvoieCommande(const commande: ansiString);
      Procedure EnvoieCommandeOPC(const commande : ansiString);
      function GetFloat(const commande: ansiString): double;
      function GetInt(const commande: ansiString; defaut: integer): integer;
      procedure SetMode(Acq: Tmode);
   end;

var
   FormMain:     TFormMain;
   ArretLecture: boolean;

implementation

uses about, comCfg, curve, multim, tds200dde, Bode, graphe;

{$R *.DFM}

const
   terminateur = crLF;
//   terminateur = crCR;
   timeOutTDS  = 27; { 27 ticks soit 1,5 seconde }

procedure TformMain.AppActivate(Sender: TObject);
begin
   if Oscillo<>oNone then
      case AcquisitionEnCours of
         mMult: MultimetreForm.OnActivate(Sender);
         mOsc: OscilloForm.OnActivate(Sender);
         mBode: BodeForm.OnActivate(Sender);
      end;
end;

procedure TFormMain.ExitItemClick(Sender: TObject);
begin
   Close;
end;

procedure TFormMain.AboutItemClick(Sender: TObject);
begin
   AboutBox.ShowModal;
end;

procedure TFormMain.FormCreate(Sender: TObject);
var
   Ini: Tinifile;
begin
   oscillo := oNone;
   OnChaineAvail := nil;
   AcquisitionEnCours := mNone;
   Application.OnActivate := AppActivate;
   Application.OnHint := ShowHint;
   Application.ShowHint := True;
   Ini := TiniFile.Create(NomFichierIni);
   voieSerie.ComNumber := Ini.ReadInteger('General', 'COMi', 1);
   Ini.Free;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
   Ini: TInifile;
begin
   if (oscillo=oSerie) and (voieSerie.Tracing=tlOn) then begin
      VoieSerie.tracing := tlDump;
      VoieSerie.logging := tlDump;
   end;
   try
      Ini := TiniFile.Create(NomFichierIni);
      Ini.WriteInteger('General', 'COMi', voieSerie.ComNumber);
      Ini.Free;
   except
   end;
end;

function TFormMain.RS232Present : boolean;
var  j: integer;
begin
      Result := IsPortAvailable(voieSerie.comNumber);
      if not Result then begin
         for j := 1 to MaxComHandles do
            if IsPortAvailable(j) then begin
               VoieSerie.ComNumber := j;
               result := true;
               break;
            end;
      end;
end;

procedure TFormMain.ResetOscillo;
const
   StrNoOsc = 'Oscillo non branch ou non reconnu';
   StrPb = 'Problme voie srie';

   procedure Verif340;
   var pos340 : integer;
   begin
         pos340 := pos('340', IdentiteScope);
         if pos340 = 0 then pos340 := pos('360', IdentiteScope);
         if pos340 = 0 then pos340 := pos('380', IdentiteScope);
         isTDS340 := pos340 > 0;
   end;

   procedure TestSefram;
   begin
      try
         IdentiteScope  := GetString('*idn?'); // Identifie the scope
         if Pos('SEFRAM', IdentiteScope)<>0 then begin
            NomScope := IdentiteScope;
            Caption  := 'Acquisition avec Sefram 5150';
            oscillo := oSerie;
         end
         else begin
            ShowMessage(StrNoOsc);
            VoieSerie.Open := False;
            Caption := StrNoOsc;
         end;
      except
         VoieSerie.Open := False;
         ShowMessage(StrPb);
         Caption := StrPb;
      end;
   end;

   procedure TestTektro;
   var
      debut, fin : integer;
   begin
      try
      IdentiteScope := GetString('ID?'); // Identifie the scope
      if  (Pos('TEK', IdentiteScope) <> 0) or
          (Pos('TDS', IdentiteScope) <> 0)
// TDS 2002 rpond
// ID[20]TEK/TDS[20]2002,CF:91.1CT,FV:v2.12[20]TDS2CM:CMV:v1.04[0A]
      then begin
         debut := pos('/', IdentiteScope);
         fin := pos(',', IdentiteScope);
         verif340;
         NomScope := copy(IdentiteScope, succ(debut), fin - debut - 1);
         Caption  := 'Acquisition avec Tektronix ' + nomScope;
         oscillo := oSerie;
     end;
     except
     end;
   end;

begin
   oscillo := oNone;
   if (oscillo=oNone) and rs232Present then begin
      try
{$IFDEF DEBUG}
   VoieSerie.Tracing := tlOn;
   VoieSerie.Logging := tlOn;
{$ENDIF}
      voieSerie.Open := True;
      TestTektro;
      if (oscillo=oNone) then TestSefram;
      except
         VoieSerie.Open := False;
         ShowMessage(StrPb);
         Caption := StrPb;
         oscillo := oNone;
      end;
   end; // rs232Present
   setMode(mNone);
end;

procedure TFormMain.SetMode(Acq: Tmode);
begin
   if Acq<>mNone then
      EnvoieCommande('HEAD OFF');
   if (Acq in [mMult, mBode]) and
      not (AcquisitionEnCours in [mMult, mBode]) and
      (pos('XY',getString('DIS:FORM?'))>0) then
      EnvoieCommande('DIS:FORM YT');
   if (Acq=mOsc) and
      (AcquisitionEnCours <> mOsc) then begin
      EnvoieCommande('DAT:ENC RPB'); { binaire positif }
      EnvoieCommande('DAT:WID 1');  { 1 octet }
      EnvoieCommande('DAT:STAR 1'); { depuis le dbut }
      EnvoieCommande('DAT:STOP 2500'); { jusqu' la fin }
   end;
   if (Acq=mOsc) and (OscilloForm=nil) then
      OscilloForm := ToscilloForm.Create(self);
   if (Acq=mMult) and (MultimetreForm=nil) then
      MultimetreForm := TmultimetreForm.Create(self);
   if (Acq=mBode) and (BodeForm=nil) then
      BodeForm := TBodeForm.Create(self);
   OscBtn.Visible  := (Oscillo<>oNone) and not (Acq in [mMult, mBode]);
   MultBtn.Visible := (Oscillo<>oNone) and not (Acq in [mOsc, mBode]);
   BodeBtn.Visible := (Oscillo<>oNone) and not (Acq in [mOsc, mMult]);
   AcquisitionEnCours := Acq;
   case Acq of
        mOsc : OscilloForm.Show;
        mMult : MultimetreForm.Show;
        mBode :  BodeForm.Show;
   end;
end;

procedure TFormMain.ResetOscilloClick(Sender: TObject);
begin
   voieSerie.Open := False;
   if RS232Present then ComCfgDlg.ShowModal;
   ResetOscillo;
end;

procedure TFormMain.FormActivate(Sender: TObject);
begin
   if Oscillo=oNone then
      ResetOscillo;
   if Oscillo=oNone then
      ResetOscilloClick(Sender);
end;

function TFormMain.GetString(const Commande: ansiString): ansiString;

   function GetStringRs232: ansiString;
   var
      ChaineLue: ansiString;
      Timer: EventTimer;
      C: ansiChar;
      avecLF: boolean;
   begin
      GetStringRS232 := '';
      EnvoieCommande(Commande);
      newTimer(timer, timeOutTDS);
      chaineLue := '';
      avecLF := False;
      repeat
         if voieSerie.charReady then begin
            C := voieSerie.GetChar;
            if C >= ' ' then
               ChaineLue := chaineLue + C;
            avecLF := C in [crCR, crLF];
         end
         else begin
            voieSerie.processCommunications;
            application.ProcessMessages;
         end;
      until avecLF or timerExpired(timer);
      if avecLF then
         GetStringRS232 := chaineLue
      else
         raise EserieError.Create(commande);
   end;

begin
     if voieSerie.open
         then result := getStringRs232
         else result := ''
end;

procedure TFormMain.EnvoieCommande(const Commande: ansiString);
begin
     if voieSerie.open then begin
         VoieSerie.FlushInBuffer;
         VoieSerie.OutPut := Commande + terminateur;
      end;
   delayTicks(2, False);
end;

Procedure TFormMain.EnvoieCommandeOPC(const Commande : ansiString);

Procedure OPCRS232;
var C : ansiChar;
    Timer : EventTimer;
begin
    VoieSerie.FlushInBuffer;
    newTimer(timer,18); // 18 ticks = 1 seconde
    VoieSerie.OutPut := Commande+terminateur;
    VoieSerie.OutPut := '*OPC?'+terminateur;
    C := #0;
    repeat
        if voieSerie.charReady then C := voieSerie.GetChar;
    until (C<>#0) or timerExpired(timer);
end;

begin
      if voieSerie.open then OPCRS232
end;

function TFormMain.GetFloat(const Commande: ansiString): double;
var
   ChaineLue: ansiString;
begin
   try
      chaineLue := getString(commande);
      if chaineLue = '' then begin
         Result := NombreTimeOut;
         exit;
      end;
// direct avec Head OFF
      Result := StrToFloatWin(chaineLue);
      if abs(Result) > 1e37 then
         Result := NombreErreur;
   except
      Result := NombreErreur
   end;
end;

function TFormMain.GetInt(const Commande: ansiString; defaut: integer): integer;
var
   chaineLue: ansiString;
begin
   try
   chaineLue := GetString(commande);
   GetInt := StrToInt(chaineLue);
   except
      GetInt := 1;
   end;
end;

procedure TFormMain.ShowHint(Sender: TObject);
begin
   if Application.Hint = '' then
      case AcquisitionEnCours of
         mOsc: OscilloForm.Caption := 'Lecture des courbes : ' + NomScope;
         else AidePanel.Caption := IdentiteScope;
      end
   else
      case AcquisitionEnCours of
         mMult: MultimetreForm.Caption := Application.Hint;
         mOsc: OscilloForm.Caption := Application.Hint;
         mBode: BodeForm.StatusBar.simpleText := Application.Hint;
         else AidePanel.Caption := Application.Hint;
      end;
end;

procedure TFormMain.OscBtnClick(Sender: TObject);
begin
   setMode(mOsc);
end;

procedure TFormMain.MultBtnClick(Sender: TObject);
begin
   setMode(mMult);
end;

procedure TFormMain.BodeBtnClick(Sender: TObject);
begin
   setMode(mBode);
end;

end.


